from twindb_infrastructure.config import TWINDB_INFRA_CONFIG
from twindb_infrastructure.config.credentials import Credentials, \
    CredentialsException


class CloudFlareCredentials(Credentials):

    cloudflare_email = None
    cloudflare_auth_key = None

    def __init__(self, config_path=TWINDB_INFRA_CONFIG):
        super(CloudFlareCredentials, self).__init__(config_path=config_path)
        if not self.config.has_section('cloudflare'):
            raise CredentialsException('There is no cloudflare '
                                       'section in config %s'
                                       % self.config_path)
        for option in self.config.options('cloudflare'):
            value = self.config.get('cloudflare', option).strip('"\'')
            setattr(self, option, value)



#
# Copyright (C) 2010-2012 Vinay Sajip.
# All rights reserved. Licensed under the new BSD license.
#
import ctypes
import logging
import logging.handlers
import os
import sys


class ColorizingStreamHandler(logging.StreamHandler):
    def __init__(self):
        logging.StreamHandler.__init__(self, sys.stdout)

    # color names to indices
    color_map = {
        'black': 0,
        'red': 1,
        'green': 2,
        'yellow': 3,
        'blue': 4,
        'magenta': 5,
        'cyan': 6,
        'white': 7,
    }

    # levels to (background, foreground, bold/intense)
    if os.name == 'nt':
        level_map = {
            logging.DEBUG: (None, 'blue', True),
            logging.INFO: (None, 'white', False),
            logging.WARNING: (None, 'yellow', True),
            logging.ERROR: (None, 'red', True),
            logging.CRITICAL: ('red', 'white', True),
        }
    else:
        level_map = {
            logging.DEBUG: (None, 'blue', False),
            logging.INFO: (None, 'green', False),
            logging.WARNING: (None, 'yellow', False),
            logging.ERROR: (None, 'red', False),
            logging.CRITICAL: ('red', 'white', True),
        }
    csi = '\x1b['
    reset = '\x1b[0m'

    @property
    def is_tty(self):
        isatty = getattr(self.stream, 'isatty', None)
        return isatty and isatty()

    def emit(self, record):
        try:
            message = self.format(record)
            stream = self.stream
            if not self.is_tty:
                stream.write(message)
            else:
                self.output_colorized(message)
            stream.write(getattr(self, 'terminator', '\n'))
            self.flush()
        except (KeyboardInterrupt, SystemExit):
            raise
        # except:
        #    self.handleError(record)

    if os.name != 'nt':
        def output_colorized(self, message):
            self.stream.write(message)
    else:
        import re
        ansi_esc = re.compile(r'\x1b\[((?:\d+)(?:;(?:\d+))*)m')

        nt_color_map = {
            0: 0x00,    # black
            1: 0x04,    # red
            2: 0x02,    # green
            3: 0x06,    # yellow
            4: 0x01,    # blue
            5: 0x05,    # magenta
            6: 0x03,    # cyan
            7: 0x07,    # white
        }

        def output_colorized(self, message):
            parts = self.ansi_esc.split(message)
            write = self.stream.write
            h = None
            fd = getattr(self.stream, 'fileno', None)
            if fd is not None:
                fd = fd()
                if fd in (1, 2):  # stdout or stderr
                    h = ctypes.windll.kernel32.GetStdHandle(-10 - fd)
            while parts:
                text = parts.pop(0)
                if text:
                    write(text)
                if parts:
                    params = parts.pop(0)
                    if h is not None:
                        params = [int(p) for p in params.split(';')]
                        color = 0
                        for p in params:
                            if 40 <= p <= 47:
                                color |= self.nt_color_map[p - 40] << 4
                            elif 30 <= p <= 37:
                                color |= self.nt_color_map[p - 30]
                            elif p == 1:
                                color |= 0x08  # foreground intensity on
                            elif p == 0:       # reset to default color
                                color = 0x07
                            else:
                                pass           # error condition ignored
                        ctypes.windll.kernel32.SetConsoleTextAttribute(h,
                                                                       color)

    def colorize(self, message, record):
        if record.levelno in self.level_map:
            bg, fg, bold = self.level_map[record.levelno]
            params = []
            if bg in self.color_map:
                params.append(str(self.color_map[bg] + 40))
            if fg in self.color_map:
                params.append(str(self.color_map[fg] + 30))
            if bold:
                params.append('1')
            if params:
                message = ''.join((self.csi, ';'.join(params),
                                   'm', message, self.reset))
        return message

    def format(self, record):
        message = logging.StreamHandler.format(self, record)
        if self.is_tty:
            # Don't colorize any traceback
            parts = message.split('\n', 1)
            parts[0] = self.colorize(parts[0], record)
            message = '\n'.join(parts)
        return message


def setup_logging(logger, debug=False):

    fmt_str = "%(asctime)s: %(levelname)s:" \
              " %(module)s.%(funcName)s():%(lineno)d: %(message)s"

    # console_handler = logging.StreamHandler()
    console_handler = ColorizingStreamHandler()
    console_handler.setFormatter(logging.Formatter(fmt_str))
    logger.handlers = []
    logger.addHandler(console_handler)
    if debug:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.INFO)




def main():
    root = logging.getLogger()
    root.setLevel(logging.DEBUG)
    root.addHandler(ColorizingStreamHandler())
    logging.debug('DEBUG')
    logging.info('INFO')
    logging.warning('WARNING')
    logging.error('ERROR')
    logging.critical('CRITICAL')

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
TwinDB Amazon infrastructure
"""
import logging
import boto3
from botocore.exceptions import ClientError
import click
from twindb_infrastructure.clogging import setup_logging
from twindb_infrastructure.config.config import TWINDB_INFRA_CONFIG, Config, \
    ConfigException


class TwinDBInfraException(Exception):
    pass


class ConfigPath(object):
    config_path = None

pass_path = click.make_pass_decorator(ConfigPath, ensure=True)
log = logging.getLogger(__name__)


def parse_config(path):
    """Parse TwinDB Infrastructure config"""
    try:
        return Config(path)
    except ConfigException as err:
        log.error(err)
        exit(-1)


@click.group()
@click.option('--config', default=TWINDB_INFRA_CONFIG,
              help='Config file')
@click.option('--debug', is_flag=True, default=False,
              help='Print debug messages')
@pass_path
def main(c_path, config, debug):
    """
    Console script to work with TwinDB Amazon Infrastructure
    """
    setup_logging(log, debug=debug)
    log.debug('Using config %s' % config)
    c_path.config_path = config


@main.command()
@click.option('--tags', is_flag=True, help='Show instance tags')
@pass_path
def show(c_path, tags):
    """List TwinDB servers"""
    parse_config(c_path.config_path)

    client = boto3.client('ec2')
    response = client.describe_instances()
    for reservation in response['Reservations']:
        for instance in reservation['Instances']:
            print(instance['InstanceId'])
            if tags:
                for tag in instance['Tags']:
                    print("\t%s: %s" % (tag['Key'], tag['Value']))


@main.command()
@click.argument('instance-id')
@pass_path
def terminate(c_path, instance_id):
    """Terminate Amazon instance"""
    parse_config(c_path.config_path)

    try:
        ec2 = boto3.resource('ec2')
        ec2.instances.filter(InstanceIds=[instance_id]).terminate()

    except ClientError as err:
        log.error(err)


@main.command()
@click.argument('instance-id')
@pass_path
def stop(c_path, instance_id):
    """Stop Amazon instance"""
    parse_config(c_path.config_path)

    try:
        ec2 = boto3.resource('ec2')
        ec2.instances.filter(InstanceIds=[instance_id]).stop()

    except ClientError as err:
        log.error(err)


@main.command()
@click.argument('instance-id')
@pass_path
def start(c_path, instance_id):
    """Start Amazon instance"""
    parse_config(c_path.config_path)

    try:
        ec2 = boto3.resource('ec2')
        ec2.instances.filter(InstanceIds=[instance_id]).start()

    except ClientError as err:
        log.error(err)

if __name__ == "__main__":
    pass



# -*- coding: utf-8 -*-

import click


@click.command()
def main(args=None):
    """Console script for twindb_infrastructure"""
    click.echo("Replace this message by putting your code into "
               "twindb_infrastructure.cli.main")
    click.echo("See click documentation at http://click.pocoo.org/")


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'Click>=6.0',
    'boto3'
]

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    name='twindb-infrastructure',
    version='0.1.3',
    description="TwinDB Infrastructure is a collection of everything"
                " to manage TwinDB infrastructure",
    long_description=readme + '\n\n' + history,
    author="TwinDB Development Team",
    author_email='dev@twindb.com',
    url='https://github.com/twindb/twindb_infrastructure',
    packages=find_packages(),
    package_dir={'twindb_infrastructure':
                 'twindb_infrastructure'},
    entry_points={
        'console_scripts': [
            'twindb_aws=twindb_infrastructure.twindb_aws:main'
        ]
    },
    include_package_data=True,
    install_requires=requirements,
    zip_safe=False,
    keywords='twindb_infrastructure',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements
)
